--[==[

Copyright ©2019 Samuel Thomas Pain

The contents of this addon, excluding third-party resources, are
copyrighted to their authors with all rights reserved.

This addon is free to use and the authors hereby grants you the following rights:

1. 	You may make modifications to this addon for private use only, you
    may not publicize any portion of this addon.

2. 	Do not modify the name of this addon, including the addon folders.

3. 	This copyright notice shall be included in all copies or substantial
    portions of the Software.

All rights not explicitly addressed in this license are reserved by
the copyright holders.

]==]--


local name, GHC = ...

GHC.Functions = {}

--prints addon message to chat window using addon formatting
function GHC.Functions.PrintMessage(msg)
    DEFAULT_CHAT_FRAME:AddMessage("|cff55ae00Guild Helper Classic: |r"..msg)
end

--debug function to find event args
function GHC.Functions.GetArgs(...)
	for i=1, select("#", ...) do
		arg = select(i, ...)
		print(i.." "..tostring(arg))
	end
end

--set a frame to be moveable
function GHC.Functions.MakeFrameMove(frame)
	frame:SetMovable(true)
	frame:EnableMouse(true)
	frame:RegisterForDrag("LeftButton")
	frame:SetScript("OnDragStart", frame.StartMoving)
	frame:SetScript("OnDragStop", frame.StopMovingOrSizing)
end

--called 1 sec after addon loaded event allowing for GUIDs
function GHC.Functions.Load()

    if GhcDb == nil then
        GhcDb = {}
    end
    if GhcDb[UnitGUID('player')] == nil then
        GhcDb[UnitGUID('player')] = { GuildDb = {}, GuildDbBackUp = {}, Settings = { Role = nil } }
        GHC.Functions.PrintMessage('created settings for '..UnitName('player'))
    else
        GHC.Functions.PrintMessage('loaded settings for '..UnitName('player'))
    end

    GHC.Functions.MakeFrameMove(GHC.UI.ParentFrame)
    
end

--scans the guild roster, creates a backup db which is checked against to update player info after roster scan
function GHC.Functions.ScanGuildRoster()

    --create a back up of current guild db
    if GhcDb[UnitGUID('player')].GuildDb ~= nil then
        --wipe current backup - all info will be old or in current db
        GhcDb[UnitGUID('player')].GuildDbBackUp = {}
        for k, v in ipairs(GhcDb[UnitGUID('player')].GuildDb) do
            GhcDb[UnitGUID('player')].GuildDbBackUp[k] = v
        end
    end

    --wipe guild db
    GhcDb[UnitGUID('player')].GuildDb = {}
    local TotalMembers, OnlineMemebrs, _ = GetNumGuildMembers()
    local race, gender, prof1Name, prof1Level, prof2Name, prof2Level, role = nil, nil, nil, nil, nil, nil, 7
	for i = 1, TotalMembers do
		local name, rankName, rankIndex, level, classDisplayName, zone, publicNote, officerNote, isOnline, status, class, achievementPoints, achievementRank, isMobile, canSoR, repStanding, guid = GetGuildRosterInfo(i)
        name = string.sub(name, 0, (string.find(name, '-') - 1))
        --if we have a back up check against it for existing data
        if GhcDb[UnitGUID('player')].GuildDbBackUp ~= nil then
            for k, member in ipairs(GhcDb[UnitGUID('player')].GuildDbBackUp) do
                if member.GUID == guid then
                    --print('found existing member')
                    race = member.Race
                    gender = member.Gender
                    prof1Name, prof1Level = member.Profession1Name, member.Profession1Level
                    prof2Name, prof2Level = member.Profession2Name, member.Profession2Level
                    role = member.Role
                end
            end
        end
        GhcDb[UnitGUID('player')].GuildDb[i] = { GUID = guid, Name = name, Level = level, Class = class, Race = race, Gender = gender, Profession1Name = prof1Name, Profession1Level = prof1Level, Profession2Name = prof2Name, Profession2Level = prof2Level, Role = role }
    end

    --clear current icons & text from listview
    for i = 1, 10 do
        GHC.GuildRosterListViewItems[i].RaceIcon.Texture:SetTexture(nil)
        GHC.GuildRosterListViewItems[i].ClassIcon.Texture:SetTexture(nil)
        GHC.GuildRosterListViewItems[i].Name:SetText('')
        GHC.GuildRosterListViewItems[i].Level:SetText('')
        GHC.GuildRosterListViewItems[i].RoleText:SetText('')
        --profession data is cleared/nil in next loop
    end

    --reset listview scroll position
    GHC.UI.RosterScrollBar:SetValue(1)
    --update listview using first 10 members - no sort function added yet
    for i = 1, 10 do
        if GhcDb[UnitGUID('player')].GuildDb[i].Gender ~= nil and GhcDb[UnitGUID('player')].GuildDb[i].Race ~= nil then
            GHC.GuildRosterListViewItems[i].RaceIcon.Texture:SetTexture(GHC.Db.RaceIcons[GhcDb[UnitGUID('player')].GuildDb[i].Gender][GhcDb[UnitGUID('player')].GuildDb[i].Race])
        end
        GHC.GuildRosterListViewItems[i].ClassIcon.Texture:SetTexture(tostring("Interface/Addons/GuildHelperClassic/ClassIcons/"..GhcDb[UnitGUID('player')].GuildDb[i].Class))
        GHC.GuildRosterListViewItems[i].Name:SetText(GhcDb[UnitGUID('player')].GuildDb[i].Name)
        GHC.GuildRosterListViewItems[i].Level:SetText(GhcDb[UnitGUID('player')].GuildDb[i].Level)
        if GhcDb[UnitGUID('player')].GuildDb[i].Role ~= nil then
            GHC.GuildRosterListViewItems[i].RoleText:SetText(GHC.Db.RoleIconTextString[tonumber(GhcDb[UnitGUID('player')].GuildDb[i].Role)])
        else
            GHC.GuildRosterListViewItems[i].RoleText:SetText(GHC.Db.RoleIconTextString[0])
        end
        --profession 1
        if GhcDb[UnitGUID('player')].GuildDb[i].Profession1Name ~= nil and GhcDb[UnitGUID('player')].GuildDb[i].Profession1Level ~= nil then
            GHC.GuildRosterListViewItems[i].Profession1Icon.Texture:SetTexture(GHC.Db.ProfessionIcons[GhcDb[UnitGUID('player')].GuildDb[i].Profession1Name])
            GHC.GuildRosterListViewItems[i].Profession1 = tostring(GhcDb[UnitGUID('player')].GuildDb[i].Profession1Name..' '..GhcDb[UnitGUID('player')].GuildDb[i].Profession1Level)
        else
            GHC.GuildRosterListViewItems[i].Profession1Icon.Texture:SetTexture(nil)
            GHC.GuildRosterListViewItems[i].Profession1 = ''
        end
        --profession 2
        if GhcDb[UnitGUID('player')].GuildDb[i].Profession2Name ~= nil and GhcDb[UnitGUID('player')].GuildDb[i].Profession2Level ~= nil then
            GHC.GuildRosterListViewItems[i].Profession2Icon.Texture:SetTexture(GHC.Db.ProfessionIcons[GhcDb[UnitGUID('player')].GuildDb[i].Profession2Name])
            GHC.GuildRosterListViewItems[i].Profession2 = tostring(GhcDb[UnitGUID('player')].GuildDb[i].Profession2Name..' '..GhcDb[UnitGUID('player')].GuildDb[i].Profession2Level)
        else
            GHC.GuildRosterListViewItems[i].Profession2Icon.Texture:SetTexture(nil)
            GHC.GuildRosterListViewItems[i].Profession2 = ''
        end
    end

    --update the class summary, reset counts first then loop db and make new count > sort > update UI
    for k, class in pairs(GHC.Db.ClassCount) do
        class.Count = 0
    end
    local totalMembers = 0
    for i, member in ipairs(GhcDb[UnitGUID('player')].GuildDb) do
        for k, class in ipairs(GHC.Db.ClassCount) do
            if member.Class == class.Class then
                class.Count = class.Count + 1
                totalMembers = totalMembers + 1
            end
        end
    end
    table.sort(GHC.Db.ClassCount, function(a, b) return a.Count > b.Count end)
    for i, class in pairs(GHC.Db.ClassCount) do
        GHC.UI.ClassSummaryBars[i].StatusBar:SetValue((tonumber(class.Count) / tonumber(totalMembers)) * 100.0)
        GHC.UI.ClassSummaryBars[i].StatusBar:SetStatusBarColor(GHC.Db.ClassColours[class.Class].r, GHC.Db.ClassColours[class.Class].g, GHC.Db.ClassColours[class.Class].b)
        GHC.UI.ClassSummaryBars[i].Icon.Texture:SetTexture(tostring("Interface/Addons/GuildHelperClassic/ClassIcons/"..class.Class))
        GHC.UI.ClassSummaryBars[i].Text:SetText(tostring(class.Count.." [ "..string.format("%.1f", (tonumber(class.Count) / tonumber(totalMembers)) * 100.0).."% ]"))
    end
end

function GHC.Functions.GuildRosterScrollBarChanged(value)
    local v = math.ceil(value)
    GHC.UI.RosterScrollBar:SetMinMaxValues(1, math.ceil(#GhcDb[UnitGUID('player')].GuildDb/10))
    local i = 1
    for i = 1, 10 do
        GHC.GuildRosterListViewItems[i].RaceIcon.Texture:SetTexture(nil)
        GHC.GuildRosterListViewItems[i].ClassIcon.Texture:SetTexture(nil)
        GHC.GuildRosterListViewItems[i].Name:SetText('')
        GHC.GuildRosterListViewItems[i].Level:SetText('')
        GHC.GuildRosterListViewItems[i].RoleText:SetText('')
    end

    --use i to access the listview item and k to access the guild db object
    for k, member in ipairs(GhcDb[UnitGUID('player')].GuildDb) do			
        if k < ((v * 10) + 1) and k > ((v - 1) * 10) then
            if GhcDb[UnitGUID('player')].GuildDb[k].Gender ~= nil and GhcDb[UnitGUID('player')].GuildDb[k].Race ~= nil then
                GHC.GuildRosterListViewItems[i].RaceIcon.Texture:SetTexture(GHC.Db.RaceIcons[GhcDb[UnitGUID('player')].GuildDb[k].Gender][GhcDb[UnitGUID('player')].GuildDb[k].Race])
            else
                GHC.GuildRosterListViewItems[i].RaceIcon.Texture:SetTexture(nil) --134400) --532) --607721)
            end
            if GhcDb[UnitGUID('player')].GuildDb[k].Role ~= nil then
                GHC.GuildRosterListViewItems[i].RoleText:SetText(GHC.Db.RoleIconTextString[tonumber(GhcDb[UnitGUID('player')].GuildDb[k].Role)])
            else
                GHC.GuildRosterListViewItems[i].RoleText:SetText(GHC.Db.RoleIconTextString[0])
            end
            GHC.GuildRosterListViewItems[i].ClassIcon.Texture:SetTexture(tostring("Interface/Addons/GuildHelperClassic/ClassIcons/"..member.Class))
            GHC.GuildRosterListViewItems[i].Name:SetText(member.Name)
            GHC.GuildRosterListViewItems[i].Level:SetText(member.Level)
            --profession 1
            if GhcDb[UnitGUID('player')].GuildDb[k].Profession1Name ~= nil and GhcDb[UnitGUID('player')].GuildDb[k].Profession1Level ~= nil then
                GHC.GuildRosterListViewItems[i].Profession1Icon.Texture:SetTexture(GHC.Db.ProfessionIcons[GhcDb[UnitGUID('player')].GuildDb[k].Profession1Name])
                GHC.GuildRosterListViewItems[i].Profession1 = tostring(GhcDb[UnitGUID('player')].GuildDb[k].Profession1Name..' '..GhcDb[UnitGUID('player')].GuildDb[k].Profession1Level)
            else
                GHC.GuildRosterListViewItems[i].Profession1Icon.Texture:SetTexture(nil)
                GHC.GuildRosterListViewItems[i].Profession1 = ''
            end
            --profession 2
            if GhcDb[UnitGUID('player')].GuildDb[k].Profession2Name ~= nil and GhcDb[UnitGUID('player')].GuildDb[k].Profession2Level ~= nil then
                GHC.GuildRosterListViewItems[i].Profession2Icon.Texture:SetTexture(GHC.Db.ProfessionIcons[GhcDb[UnitGUID('player')].GuildDb[k].Profession2Name])
                GHC.GuildRosterListViewItems[i].Profession2 = tostring(GhcDb[UnitGUID('player')].GuildDb[k].Profession2Name..' '..GhcDb[UnitGUID('player')].GuildDb[k].Profession2Level)
            else
                GHC.GuildRosterListViewItems[i].Profession2Icon.Texture:SetTexture(nil)
                GHC.GuildRosterListViewItems[i].Profession2 = ''
            end
            i = i + 1
        end
    end
end

function GHC.Functions.CreateMyDataString()

    local player = { Name = nil, Level = UnitLevel('player'), Class = nil, Race = nil, Gender = nil, Profession1Name = '-', Profession1Level = 0, Profession2Name = '-', Profession2Level = 0}

    _, class, _, race, gender, player.Name, _ = GetPlayerInfoByGUID(UnitGUID('player'))
    player.Race = string.upper(race)
    player.Class = string.upper(class)

    if gender == 3 then
        player.Gender = "FEMALE"
    else
        player.Gender = "MALE"
    end

    -- talent data isn't gathered at the moment and the intention is to avoid using it, instead players select their role(s) regardless of class/talents

    --will look at adding secondary prof data in a later release depending on community feedback, high level cooking is useful for food buffs and maybe useful to know
    for s = 1, GetNumSkillLines() do
		local skill, _, _, level, _, _, _, _, _, _, _, _, _ = GetSkillLineInfo(s)
		if skill == 'Fishing' then 
			--player.Fishing = level
		elseif skill == 'Cooking' then
			--player.Cooking = level
		elseif skill == 'First Aid' then
			--player.FirstAid = level
		else
			for k, prof in ipairs(GHC.Db.Professions) do
				if skill == prof.Name then
					if player.Profession1Name == '-' then
						player.Profession1Name = skill
						player.Profession1Level = level
					elseif player.Profession2Name == '-' then
						player.Profession2Name = skill
						player.Profession2Level = level
					end
				end
			end
		end
    end
    local role = 7 --default as any/all roles
    if GhcDb[UnitGUID('player')].Settings.Role ~= nil then
        role = GhcDb[UnitGUID('player')].Settings.Role
    end
    dataString = tostring(UnitGUID('player')..':'..player.Name..':'..player.Class..':'..player.Race..':'..player.Gender..':'..player.Level..':'..player.Profession1Name..':'..player.Profession1Level..':'..player.Profession2Name..':'..player.Profession2Level..':'..role)
    return dataString
end

function GHC.Functions.SendMyData()
    local sent = C_ChatInfo.SendAddonMessage('ghc-playerdata', GHC.Functions.CreateMyDataString(), "GUILD")
    if sent == true then
        --print('data sent')
    end
end

function GHC.Functions.ParsePlayerData(dataSent)
	local i = 1
	local dataSent = {}
	for d in string.gmatch(dataString, '[^:]+') do
        dataSent[i] = d
		i = i + 1
    end
    local addPlayer = true
    --this data is copied into the backup when a new roster scan occurs, its used to then update the new scan    
    if GhcDb[UnitGUID('player')].GuildDb ~= nil then
        for k, member in ipairs(GhcDb[UnitGUID('player')].GuildDb) do
            if member.GUID == dataSent[1] then
                member.Name = dataSent[2]
                member.Class = dataSent[3]
                member.Race = dataSent[4]
                member.Gender = dataSent[5]
                member.Level = dataSent[6]
                member.Profession1Name = dataSent[7]
                member.Profession1Level = dataSent[8]
                member.Profession2Name = dataSent[9]
                member.Profession2Level = dataSent[10]
                member.Role = dataSent[11]
                --print('updated', dataSent[2])
                addPlayer = false
            end
        end
        --if player wasn't in db add new
        if addPlayer == true then
            table.insert(GhcDb[UnitGUID('player')].GuildDb, {GUID == dataSent[1], Name = dataSent[2], Class = dataSent[3], Race = dataSent[4], Gender = dataSent[5], Level = dataSent[6], Profession1Name = dataSent[7], Profession1Level = dataSent[8], Profession2Name = dataSent[9], Profession2Level = dataSent[10], Role = dataSent[11]} )
        end
    end
    GHC.Functions.ScanGuildRoster()
end



--print('loaded scripts')